#!/bin/bash -e
#
# $Id: build-updates.sh 18688 2021-08-08 23:51:28Z NiLuJe $
#

HACKDIR="USBNetwork"
HACKNAME="usbnet"
PKGNAME="usbnetwork"
PKGVER="0.57.N"

# Setup KindleTool packaging metadata flags to avoid cluttering the invocations
PKGREV="$(svnversion -c .. | awk '{print $NF}' FS=':' | tr -d 'P')"
KT_PM_FLAGS=( "-xPackageName=${HACKDIR}" "-xPackageVersion=${PKGVER}-r${PKGREV}" "-xPackageAuthor=NiLuJe" "-xPackageMaintainer=NiLuJe" "-X" )

# Do we have kindletool (https://github.com/NiLuJe/KindleTool) in $PATH?
if kindletool help &>/dev/null ; then
	USE_KINDLETOOL="true"
fi

# We also need GNU tar
if [[ "$(uname -s)" == "Darwin" ]] ; then
	TAR_BIN="gtar"
else
	TAR_BIN="tar"
fi
if ! ${TAR_BIN} --version | grep -q "GNU tar" ; then
	echo "You need GNU tar to build this package."
	exit 1
fi

# Force use of the python tool
#USE_KINDLETOOL="false"

# Pickup our common stuff... We leave it in our staging wd so it ends up in the source package.
if [[ ! -d "../../Common" ]] ; then
	echo "The tree isn't checked out in full, missing the Common directory..."
	exit 1
fi
# LibOTAUtils
ln -f ../../Common/lib/libotautils ./libotautils
# XZ Utils
ln -f ../../Common/bin/xzdec ./xzdec
# LibKH
for common_lib in libkh ; do
	ln -f ../../Common/lib/${common_lib} ../src/${HACKNAME}/bin/${common_lib}
done


KINDLE_MODELS="k2 k2i dx dxi dxg k3g k3w k3gb"

# Archive custom directory
export XZ_DEFAULTS="-T 0"
${TAR_BIN} --hard-dereference --owner root --group root --exclude-vcs -cvJf ${HACKNAME}.tar.xz ../src/${HACKNAME} ../src/extensions

# OTAv2 MRPI only packages or not?
if [[ "${NO_MRPI_ONLY_PACKAGES}" == "true" ]] ; then
	for model in ${KINDLE_MODELS} ; do
		# Prepare our files for this specific kindle model...
		ARCH="${PKGNAME}_${PKGVER}_${model}"

		if [[ "$USE_KINDLETOOL" == "true" ]] ; then
			# Install
			kindletool create ota -d ${model} libotautils install.sh ${HACKNAME}-preinit ${HACKNAME}-init ${HACKNAME}.tar.xz xzdec Update_${ARCH}_install.bin
			# Uninstall
			kindletool create ota -d ${model} libotautils uninstall.sh Update_${ARCH}_uninstall.bin
		else
			# Build install update
			./kindle_update_tool.py m --${model} --sign ${ARCH}_install libotautils install.sh ${HACKNAME}-preinit ${HACKNAME}-init ${HACKNAME}.tar.xz xzdec

			# Build uninstall update
			./kindle_update_tool.py m --${model} --sign ${ARCH}_uninstall libotautils uninstall.sh
		fi
	done
else
	if [[ "$USE_KINDLETOOL" == "true" ]] ; then
		ARCH="${PKGNAME}_${PKGVER}_k2_dx_k3"
		# Install
		kindletool create ota2 "${KT_PM_FLAGS[@]}" -d legacy libotautils install.sh ${HACKNAME}-preinit ${HACKNAME}-init ${HACKNAME}.tar.xz xzdec Update_${ARCH}_install.bin
		# Uninstall
		kindletool create ota2 "${KT_PM_FLAGS[@]}" -d legacy -d kindle4 libotautils uninstall.sh Update_${PKGNAME}_${PKGVER}_uninstall.bin
	fi
fi

# Remove custom directory archive
rm -f ${HACKNAME}.tar.xz

# K4
if [[ "$USE_KINDLETOOL" == "true" ]] ; then
	## Go away if we don't have the Touch_hacks tree checked out...
	if [[ ! -d "../../../Touch_Hacks" ]] ; then
		echo "Skipping K4 build, we don't have the proper binaries on hand (hint: check out the Touch_Hacks tree, too)"
	else
		# NOTE: This isn't particularly pretty, and we end up bundling only one version of the binaries in the zip files, but at least the .bin files end up with the correct set of binaries ;)
		# K4/K5 binaries...
		KINDLE_MODEL_BINARIES="bin/busybox bin/dropbearmulti bin/htop bin/kindletool bin/kindle_usbnet_addr bin/lsof bin/mosh-client bin/mosh-server bin/rsync bin/scp bin/sftp bin/ssh bin/ssh-add bin/ssh-agent bin/sshfs bin/ssh-keygen bin/ssh-keyscan libexec/sftp-server libexec/ssh-keysign libexec/ssh-pkcs11-helper libexec/ssh-sk-helper sbin/sshd bin/fbgrab bin/strace bin/eu-nm bin/eu-objdump bin/eu-strings lib/libasm.so.1 lib/libdw.so.1 lib/libelf.so.1 lib/libz.so.1 lib/libpng16.so.16 lib/libncurses.so.6 lib/libtinfo.so.6 lib/libncursesw.so.6 lib/libtinfow.so.6 lib/libunwind-arm.so.8 lib/libunwind-ptrace.so.0 lib/libunwind.so.8 bin/nano bin/zsh lib/libmagic.so.1 lib/libpcre.so.1 lib/libpcreposix.so.0"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} lib/zsh/attr.so lib/zsh/cap.so lib/zsh/clone.so lib/zsh/compctl.so lib/zsh/complete.so lib/zsh/complist.so lib/zsh/computil.so lib/zsh/curses.so lib/zsh/datetime.so lib/zsh/deltochar.so lib/zsh/example.so lib/zsh/files.so lib/zsh/langinfo.so lib/zsh/mapfile.so lib/zsh/mathfunc.so lib/zsh/nearcolor.so lib/zsh/newuser.so lib/zsh/parameter.so lib/zsh/pcre.so lib/zsh/regex.so lib/zsh/rlimits.so lib/zsh/sched.so lib/zsh/stat.so lib/zsh/system.so lib/zsh/termcap.so lib/zsh/terminfo.so lib/zsh/zftp.so lib/zsh/zleparameter.so lib/zsh/zle.so lib/zsh/zprof.so lib/zsh/zpty.so lib/zsh/zselect.so lib/zsh/zutil.so lib/zsh/net/socket.so lib/zsh/net/tcp.so lib/zsh/param/private.so"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} bin/b2sum bin/dircolors bin/ag lib/libevent-2.1.so.7 lib/libevent_core-2.1.so.7 lib/libevent_extra-2.1.so.7 lib/libevent_openssl-2.1.so.7 lib/libevent_pthreads-2.1.so.7 bin/tmux bin/gdb bin/gdbserver bin/objdump bin/gprof bin/fbink bin/fbdepth"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} bin/curl lib/libcurl.so.4 bin/evtest lib/libevdev.so.2 bin/evemu-describe bin/evemu-device bin/evemu-event bin/evemu-play bin/evemu-record lib/libevemu.so.3"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} lib/libcrypto.so.1.1 lib/libssl.so.1.1"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} bin/scanelf lib/libpcre2-8.so.0 lib/libpcre2-posix.so.3 bin/less bin/lessecho bin/lesskey"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} lib/libbfd-2.37.so lib/libctf-nobfd.so.0 lib/libctf.so.0 lib/libopcodes-2.37.so"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} lib/libsmartcols.so.1 lib/libmount.so.1 lib/libblkid.so.1 sbin/blkid bin/lsblk bin/choom bin/lsipc bin/lslocks"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} bin/jq"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} lib/libexpat.so.1 lib/libzstd.so.1 lib/liblz4.so.1 lib/libarchive.so.13 bin/bsdtar"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} lib/libxxhash.so.0"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} lib/libreadline.so.8"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} bin/openssl"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} bin/lftp"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} bin/tree"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} bin/file"
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} bin/dtc"

		# Archive custom directory
		${TAR_BIN} --hard-dereference --owner root --group root --exclude-vcs -cvf ${HACKNAME}.tar ../src/${HACKNAME} ../src/extensions
		# Delete K3 binaries
		for my_bin in ${KINDLE_MODEL_BINARIES} ; do
			${TAR_BIN} --delete -vf ${HACKNAME}.tar src/${HACKNAME}/${my_bin}
		done
		# Also delete the K3 readelf...
		for my_bin in bin/readelf ; do
			${TAR_BIN} --delete -vf ${HACKNAME}.tar src/${HACKNAME}/${my_bin}
		done
		# Don't forget ltrace & eu-readelf (they're K5 only, so we don't have to delete them first)
		KINDLE_MODEL_BINARIES="${KINDLE_MODEL_BINARIES} bin/ltrace bin/eu-readelf"
		# Append K4 binaries
		for my_bin in ${KINDLE_MODEL_BINARIES} ; do
			${TAR_BIN} --hard-dereference --owner root --group root --transform "s,^Touch_Hacks/${HACKDIR}/,,S" --show-transformed-names -rvf ${HACKNAME}.tar ../../../Touch_Hacks/${HACKDIR}/src/${HACKNAME}/${my_bin}
		done
		# xz it...
		xz ${HACKNAME}.tar

		# Speaking of, we need our own xzdec binary, too!
		ln -f ../../../Touch_Hacks/Common/bin/xzdec ./xzdec

		# Prepare our files for this specific kindle models...
		ARCH=${PKGNAME}_${PKGVER}_k4

		# Install
		kindletool create ota2 "${KT_PM_FLAGS[@]}" -d kindle4 libotautils install.sh ${HACKNAME}-preinit ${HACKNAME}-init ${HACKNAME}.tar.xz xzdec Update_${ARCH}_install.bin
		# Uninstall has already been taken care of if we're building unified packages :).
		if [[ "${NO_MRPI_ONLY_PACKAGES}" == "true" ]] ; then
			# Uninstall
			kindletool create ota2 "${KT_PM_FLAGS[@]}" -d kindle4 libotautils uninstall.sh Update_${ARCH}_uninstall.bin
		fi

		# Remove custom directory archive
		rm -f ${HACKNAME}.tar.xz
	fi
fi

# Cleanup
rm -f xzdec

# Move our updates :)
mv -f *.bin ../
