#!/bin/sh
#
# $Id: install.sh 18668 2021-08-03 18:37:41Z NiLuJe $
#

# Pull libOTAUtils for logging & progress handling
[ -f ./libotautils ] && source ./libotautils


# Hack specific config (name and when to start/stop)
HACKNAME="usbnet"
PSLEVEL="62"
SLEVEL="82"
KLEVEL="09"

# Based on version 0.11 (20100124), ebs, jya
HACKVER="0.57.N"

# Directories
USBNET_BASEDIR="/mnt/us/usbnet"
USBNET_BINDIR="${USBNET_BASEDIR}/bin"
USBNET_SBINDIR="${USBNET_BASEDIR}/sbin"
USBNET_LIBEDIR="${USBNET_BASEDIR}/libexec"
USBNET_LIBDIR="${USBNET_BASEDIR}/lib"

USBNET_LOG="${USBNET_BASEDIR}/usbnetwork_install.log"

if grep -q '^Kindle 4' /etc/prettyversion.txt ; then
    IS_K4="true"
else
    IS_K4="false"
fi

# The K4 looks for usbnetwork in /usr/local/bin ...
if [ "${IS_K4}" == "true" ] ; then
    KINDLE_TESTDIR="/usr/local/bin"
else
    KINDLE_TESTDIR="/test/bin"
fi
KINDLE_USBNETBIN="${KINDLE_TESTDIR}/usbnetwork"

USBNET_USBNETBIN="${USBNET_BINDIR}/usbnetwork"

# Result codes
OK=0
ERR=${OK}

otautils_update_progressbar

# Install our hack's custom content
# But keep the user's custom content...
if [ -d /mnt/us/${HACKNAME} ] ; then
    IS_FRESH_INSTALL="false"
    logmsg "I" "install" "" "our custom directory already exists, checking if we have custom content to preserve"
    # Custom IP config
    if [ -f /mnt/us/${HACKNAME}/etc/config ] ; then
        cfg_expected_md5="161180bacc0ede7e5b6b571362dc525a f43f8b61a2c5fee35a5daf3fab2fba50 b8e6b6f041a5047de446dd8d31422c3f 78bf6a75c118bcfd426087c5656acff7 0f62f45f6ec5e170163c3e183f32af12 c21d338577295c4c48bf3457996dd2cd 3d630a6bdbeae41be8c2a2436159eeee 4836fc4a5383b38ff062afebc1da4972 10d30f97f0adbdb86869e48c01095c82"
        cfg_current_md5=$( md5sum /mnt/us/${HACKNAME}/etc/config | awk '{ print $1; }' )
        cfg_md5_match="false"
        for cur_exp_md5 in ${cfg_expected_md5} ; do
            if [ "${cfg_current_md5}" == "${cur_exp_md5}" ] ; then
                cfg_md5_match="true"
            fi
        done
        if [ "${cfg_md5_match}" != "true" ] ; then
            HACK_EXCLUDE="${HACKNAME}/etc/config"
            logmsg "I" "install" "" "found custom ip config, excluding from archive"
        fi
   fi
else
    # Remember that we're a fresh install for the K4 tweaks...
    IS_FRESH_INSTALL="true"
fi

otautils_update_progressbar

# Okay, now we can extract it. Since busybox's tar is very limited, we have to use a tmp directory to perform our filtering
logmsg "I" "install" "" "installing custom directory"
# Make sure our xzdec binary is executable first...
chmod +x ./xzdec
./xzdec ${HACKNAME}.tar.xz | tar -xvf -
# Do check if that went well
_RET=$?
if [ ${_RET} -ne 0 ] ; then
    logmsg "C" "install" "code=${_RET}" "failed to extract custom directory in tmp location"
    return 1
fi

cd src
# Make a copy of the default config...
cp -f usbnet/etc/config usbnet/etc/config.default
# And now we filter the content to preserve user's custom content
for custom_file in ${HACK_EXCLUDE} ; do
    if [ -f "./${custom_file}" ] ; then
        logmsg "I" "install" "" "preserving custom content (${custom_file})"
        rm -f "./${custom_file}"
    fi
done
# Finally, unleash our filtered dir on the live userstore
cp -af . /mnt/us/
_RET=$?
if [ ${_RET} -ne 0 ] ; then
    logmsg "C" "install" "code=${_RET}" "failure to update userstore with custom directory"
    return 1
fi
cd - >/dev/null
rm -rf src

otautils_update_progressbar

# Here we go
echo >> ${USBNET_LOG}
echo "usbnetwork v${HACKVER}, $( date )" >> ${USBNET_LOG}

otautils_update_progressbar

# Always enable USE_VOLUMD on a K4...
if grep -q '^Kindle 4' /etc/prettyversion.txt ; then
    logmsg "I" "install" "" "we're on a k4, enabling use_volumd"
    sed -e 's/USE_VOLUMD=".*"/USE_VOLUMD="true"/' -i ${USBNET_BASEDIR}/etc/config >> ${USBNET_LOG} 2>&1

    # And on fresh installs, use the Amazon defaults to be consistent with diags...
    if [ "${IS_FRESH_INSTALL}" == true ] ; then
        sed -e 's/HOST_IP=.*/HOST_IP=192\.168\.15\.201/' -i ${USBNET_BASEDIR}/etc/config >> ${USBNET_LOG} 2>&1
        sed -e 's/KINDLE_IP=.*/KINDLE_IP=192\.168\.15\.244/' -i ${USBNET_BASEDIR}/etc/config >> ${USBNET_LOG} 2>&1
    fi

    # Tweak the default config, too.
    sed -e 's/USE_VOLUMD=".*"/USE_VOLUMD="true"/' -i ${USBNET_BASEDIR}/etc/config.default >> ${USBNET_LOG} 2>&1
    sed -e 's/HOST_IP=.*/HOST_IP=192\.168\.15\.201/' -i ${USBNET_BASEDIR}/etc/config.default >> ${USBNET_LOG} 2>&1
    sed -e 's/KINDLE_IP=.*/KINDLE_IP=192\.168\.15\.244/' -i ${USBNET_BASEDIR}/etc/config.default >> ${USBNET_LOG} 2>&1
fi

otautils_update_progressbar

# Remove our deprecated content
# From v0.21.N
logmsg "I" "install" "" "removing deprecated init scripts & symlinks (v0.21.N)"
if [ -f /etc/init.d/${HACKNAME}-wifi ] ; then
    echo "/etc/init.d/${HACKNAME}-wifi exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/init.d/${HACKNAME}-wifi >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

if [ -L /etc/rc5.d/S72${HACKNAME}-wifi ] ; then
    echo "symbolic link /etc/rc5.d/S72${HACKNAME}-wifi exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/rc5.d/S72${HACKNAME}-wifi >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# From v0.22.N
logmsg "I" "install" "" "removing deprecated symlinks (v0.22.N)"
if [ -L /etc/rcS.d/S72${HACKNAME} ] ; then
    echo "symbolic link /etc/rcS.d/S72${HACKNAME} exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/rcS.d/S72${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# From v0.23.N
logmsg "I" "install" "" "removing deprecated symlinks (v0.23.N)"
if [ -L /etc/rc5.d/S99${HACKNAME} ] ; then
    echo "symbolic link /etc/rc5.d/S99${HACKNAME} exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/rc5.d/S99${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# From v0.52.N
logmsg "I" "install" "" "removing deprecated binaries (v0.52.N)"
LIST="sshd sftp-server"
for var in ${LIST} ; do
    if [ -f ${USBNET_BINDIR}/${var} ] ; then
        echo "deprecated binary ${USBNET_BINDIR}/${var} exists, deleting..." >> ${USBNET_LOG}
        rm -f ${USBNET_BINDIR}/${var}
    fi
done

otautils_update_progressbar

# From v0.52.N
logmsg "I" "install" "" "removing deprecated symlinks (v0.52.N)"
LIST="/usr/bin/scp /usr/bin/ssh"
for var in ${LIST} ; do
    if [ -L ${var} ] ; then
        echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
        DBM=$( readlink ${var} )
        if [ "${DBM}" = "${USBNET_BINDIR}/dropbearmulti" -o "${DBM}" = "/usr/sbin/dropbearmulti" ] ; then
            rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# From v0.52.N
logmsg "I" "install" "" "removing deprecated sshd symlink (v0.52.N)"
var="/usr/sbin/sshd"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/sshd" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# From v0.57.N
logmsg "I" "install" "" "removing deprecated libraries (v0.57.N)"
LIST="libncurses.so.5 libncursesw.so.5"
for var in ${LIST} ; do
    if [ -f ${USBNET_LIBDIR}/${var} ] ; then
        echo "deprecated library ${USBNET_LIBDIR}/${var} exists, deleting..." >> ${USBNET_LOG}
        rm -f ${USBNET_LIBDIR}/${var}
    fi
done

otautils_update_progressbar

logmsg "I" "install" "" "removing deprecated libraries (v0.57.N)"
LIST="libopcodes-2.34.0.so libbfd-2.34.0.so libopcodes-2.35.0.so libbfd-2.35.0.so libopcodes-2.35.1.so libbfd-2.35.1.so libopcodes-2.36.0.so libbfd-2.36.0.so libopcodes-2.36.1.so libbfd-2.36.1.so"
for var in ${LIST} ; do
    if [ -f ${USBNET_LIBDIR}/${var} ] ; then
        echo "deprecated library ${USBNET_LIBDIR}/${var} exists, deleting..." >> ${USBNET_LOG}
        rm -f ${USBNET_LIBDIR}/${var}
    fi
done

otautils_update_progressbar

# From v0.57.N
logmsg "I" "install" "" "removing deprecated zsh symlink (v0.57.N)"
var="/usr/bin/zsh"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/zsh" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Make sure our custom binaries are executable
LIST="busybox dropbearmulti htop kindletool kindle_usbnet_addr lsof mosh-client mosh-server rsync rsync-ssl scp sftp ssh ssh-add ssh-agent sshfs ssh-keygen ssh-keyscan usbnet-disable usbnet-enable usbnet-link usbnetwork fbgrab"
for var in ${LIST} ; do
    [ -x ${USBNET_BINDIR}/${var} ] || chmod +x ${USBNET_BINDIR}/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
done

otautils_update_progressbar

LIST="sshd"
for var in ${LIST} ; do
    [ -x ${USBNET_SBINDIR}/${var} ] || chmod +x ${USBNET_SBINDIR}/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
done

otautils_update_progressbar

LIST="sftp-server ssh-keysign ssh-pkcs11-helper ssh-sk-helper"
for var in ${LIST} ; do
    [ -x ${USBNET_LIBEDIR}/${var} ] || chmod +x ${USBNET_LIBEDIR}/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
done

otautils_update_progressbar

# Make sure the /test/bin directory exists
logmsg "I" "install" "" "creating the ${KINDLE_TESTDIR} directory if need be"
[ -d ${KINDLE_TESTDIR} ] || mkdir -p ${KINDLE_TESTDIR} >> ${USBNET_LOG} 2>&1 || exit ${ERR}

otautils_update_progressbar

# Setup SSH server
logmsg "I" "install" "" "installing SSH server"
LIST="/usr/sbin/dropbearmulti /usr/bin/dropbear /usr/bin/dbclient /usr/bin/dropbearkey /usr/bin/dropbearconvert /usr/bin/dbscp"
for var in ${LIST} ; do
    if [ -L ${var} ] ; then
        echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x ${var} ] ; then
            echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/dropbearmulti ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup lsof
logmsg "I" "install" "" "installing lsof"
var="/usr/sbin/lsof"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_BINDIR}/lsof ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

# Setup htop
logmsg "I" "install" "" "installing htop"
var="/usr/bin/htop"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_BINDIR}/htop ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

# Setup rsync
logmsg "I" "install" "" "installing rsync"
LIST="rsync rsync-ssl"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup mosh
logmsg "I" "install" "" "installing mosh (server)"
var="/usr/bin/mosh-server"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_BINDIR}/mosh-server ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

logmsg "I" "install" "" "installing mosh (client)"
var="/usr/bin/mosh-client"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_BINDIR}/mosh-client ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

# Setup kindletool
logmsg "I" "install" "" "installing kindletool"
var="/usr/bin/kindletool"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_BINDIR}/kindletool ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

# Setup fbgrab
logmsg "I" "install" "" "installing fbgrab"
var="/usr/bin/fbgrab"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_BINDIR}/fbgrab ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

# Setup sshd (OpenSSH)
logmsg "I" "install" "" "installing sshd"
var="/usr/sbin/sshd"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_SBINDIR}/sshd ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

# Setup ssh* & sftp* (OpenSSH)
logmsg "I" "install" "" "installing ssh (client & utils)"
LIST="scp sftp ssh ssh-add ssh-agent ssh-keygen ssh-keyscan sshfs"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup strace, ltrace & elfutils tools
logmsg "I" "install" "" "installing strace and friends"
LIST="strace eu-nm eu-objdump eu-readelf eu-strings ltrace readelf"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            # We don't ship ltrace w/ legacy packages, so make sure the binary actually exists first... (Same thing for readelf)
            if [ -f ${USBNET_BINDIR}/${var} ] ; then
                ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
            fi
        fi
    fi
done

otautils_update_progressbar

# Setup nano
logmsg "I" "install" "" "installing nano"
var="/usr/bin/nano"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_BINDIR}/nano ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

# Setup zsh
logmsg "I" "install" "" "installing zsh"
var="/bin/zsh"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_BINDIR}/zsh ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

# Setup ag
logmsg "I" "install" "" "installing ag"
var="/usr/bin/ag"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_BINDIR}/ag ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

# Setup tmux
logmsg "I" "install" "" "installing tmux"
var="/usr/bin/tmux"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_BINDIR}/tmux ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

# Setup objdump
logmsg "I" "install" "" "installing objdump"
var="/usr/bin/objdump"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_BINDIR}/objdump ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

# Setup gprof
logmsg "I" "install" "" "installing gprof"
var="/usr/bin/gprof"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) already exists, skipping..." >> ${USBNET_LOG}
else
    if [ -x ${var} ] ; then
        echo "Binary ${var} already exists, skipping..." >> ${USBNET_LOG}
    else
        ln -fs ${USBNET_BINDIR}/gprof ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
fi

otautils_update_progressbar

# Setup fbink
logmsg "I" "install" "" "installing fbink"
LIST="fbink fbdepth"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup evemu
logmsg "I" "install" "" "installing evemu"
LIST="evemu-describe evemu-device evemu-event evemu-play evemu-record"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup pax-utils
logmsg "I" "install" "" "installing pax-utils"
LIST="scanelf lddtree symtree"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup util-linux
logmsg "I" "install" "" "installing util-linux"
LIST="lsblk choom lsipc lslocks"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

LIST="blkid"
for var in ${LIST} ; do
    if [ -L /sbin/${var} ] ; then
        echo "symbolic link /sbin/${var} -> $( readlink /sbin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /sbin/${var} ] ; then
            echo "Binary /sbin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_SBINDIR}/${var} /sbin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup jq
logmsg "I" "install" "" "installing jq"
LIST="jq"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup bsdtar
logmsg "I" "install" "" "installing bsdtar"
LIST="bsdtar"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup lftp
logmsg "I" "install" "" "installing lftp"
LIST="lftp lftpget"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup tree
logmsg "I" "install" "" "installing tree"
LIST="tree"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup b2sum
logmsg "I" "install" "" "installing b2sum"
LIST="b2sum"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup file
logmsg "I" "install" "" "installing file"
LIST="file"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) already exists, skipping..." >> ${USBNET_LOG}
    else
        if [ -x /usr/bin/${var} ] ; then
            echo "Binary /usr/bin/${var} already exists, skipping..." >> ${USBNET_LOG}
        else
            ln -fs ${USBNET_BINDIR}/${var} /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        fi
    fi
done

otautils_update_progressbar

# Setup `usbNetwork command symlink
logmsg "I" "install" "" "setting up usbNetwork private command symlink"
if [ -L ${KINDLE_USBNETBIN} ] ; then
    echo "symbolic link ${KINDLE_USBNETBIN} -> $( readlink ${KINDLE_USBNETBIN} ) already exists, skipping..." >> ${USBNET_LOG}
else
    # Save normal file in case it already exists
    if [ -f ${KINDLE_USBNETBIN} ] ; then
        echo "${KINDLE_USBNETBIN} exists, saving..." >> ${USBNET_LOG}
        cp ${KINDLE_USBNETBIN} ${USBNET_USBNETBIN}-save.${HACKVER} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        rm -f ${KINDLE_USBNETBIN} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi

    # Create a link
    ln -fs ${USBNET_USBNETBIN} ${KINDLE_USBNETBIN} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# Setup the whole custom mac address stuff...
logmsg "I" "install" "" "computing custom mac address"
# Start by building the NIC part of a MAC address from our S/N
# NOTE: This isn't terribly pretty. Another idea would be to use a 24bit or 16bit hash/checksum, but they're not that common, unfortunately, and I really don't want to trim a larger checksum...
serial="$(cat < /proc/usid)"

# Loop over each char... (can't do C-style for loops...)
i=0
while [ ${i} -lt ${#serial} ] ; do
    # We only care about the device code, and the last four chars
    case "${i}" in
        2 | 3 )
            # Device code
            # NOTE: Can't do this on the K4...
            #mac="${mac}${serial:${i}:1}"

            char="$( echo -n "${serial}" | sed -re "s/(.){$i}(.)(.*?)/\2/g" )"
            # Build our mac...
            mac="${mac}${char}"
        ;;
        12 | 13 | 14 | 15 )
            # Last four chars (sanitize them, they're not in hex)
            char="$( echo -n "${serial}" | sed -re "s/(.){$i}(.)(.*?)/\2/g" )"

            # Convert to hex (full)
            # NOTE: Yep, the single quotes are key here, we want the hex representation of the char, not of an int.
            hex_char="$(printf "%X" "'${char}'")"
            # Then to int...
            int_char="$(printf "%d" "0x${hex_char}")"

            # 71 is G
            if [ ${int_char} -ge 71 ] ; then
                # Dumb it down to one char in the hex range
                dumb_int_char="$(( int_char % 16 ))"

                # And convert it to hex
                char="$(printf "%X" "${dumb_int_char}")"
            fi

            # Keep building our mac...
            mac="${mac}${char}"
        ;;
        * )
            # Dump it
        ;;
    esac

    let "i += 1"
done

echo "S/N ${serial} => NIC ${mac}" >> ${USBNET_LOG}

# Check if our nic mac looks okay...
# Arg 1 is mac to check
is_valid_mac() {
    mac="${1}"

    if printf "%X" "0x${mac}" > /dev/null 2>&1 ; then
        # It's in hex!
        if [ ${#mac} -eq 6 ] ; then
            # It's 24bit!
            return 0
        fi
    fi

    # Huh hoh...
    echo "mac ${mac} is invalid" >> ${USBNET_LOG}
    return 1
}

if is_valid_mac "${mac}" ; then
    echo "mac is valid" >> ${USBNET_LOG}
else
    # Invalid, fallback to default value
    mac="000000"
fi

# Save the computed copy
echo "custom_mac_nic='${mac}'" > ${USBNET_BASEDIR}/etc/custom_mac

otautils_update_progressbar

# Then build our tweaked kdb keyfile for volumd...
logmsg "I" "install" "" "building tweaked kdb keyfile"
cat > ${USBNET_BASEDIR}/etc/TURN_ON_NETWORKING_COMMAND << EOF
RG002
40
<DATA>
EOF
echo -n "modprobe g_ether host_addr='EE4900${mac}' dev_addr='EE1900${mac}'" >> ${USBNET_BASEDIR}/etc/TURN_ON_NETWORKING_COMMAND

## Check if our kdb keyfile looks okay
# Arg 1 is the file to check
is_kdb_keyfile_okay()
{
    kdb_key="${1}"

    if [ $(stat -c %s "${kdb_key}") -eq 81 ] ; then
        # Good :)
        return 0
    fi

    # Meep!
    echo "kdb keyfile looks wrong" >> ${USBNET_LOG}
    return 1
}

if is_kdb_keyfile_okay "${USBNET_BASEDIR}/etc/TURN_ON_NETWORKING_COMMAND" ; then
    echo "kdb keyfile looks ok" >> ${USBNET_LOG}
else
    # Looks broken, delete it
    rm -f "${USBNET_BASEDIR}/etc/TURN_ON_NETWORKING_COMMAND"
fi

otautils_update_progressbar

# Setup auto USB network startup script
logmsg "I" "install" "" "installing preinit script"
cp -f ${HACKNAME}-preinit /etc/init.d/${HACKNAME}-preinit >> ${USBNET_LOG} 2>&1 || exit ${ERR}
[ -x /etc/init.d/${HACKNAME}-preinit ] || chmod +x /etc/init.d/${HACKNAME}-preinit >> ${USBNET_LOG} 2>&1 || exit ${ERR}

otautils_update_progressbar

logmsg "I" "install" "" "installing init script"
cp -f ${HACKNAME}-init /etc/init.d/${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
[ -x /etc/init.d/${HACKNAME} ] || chmod +x /etc/init.d/${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}

otautils_update_progressbar

# The preinit script needs to start before pmond & volumd...
logmsg "I" "install" "" "creating preinit boot symlink"
if [ -L /etc/rc5.d/S${PSLEVEL}${HACKNAME}-preinit ] ; then
    echo "symbolic link /etc/rc5.d/S${PSLEVEL}${HACKNAME}-preinit already exists, skipping..." >> ${USBNET_LOG}
else
    ln -fs /etc/init.d/${HACKNAME}-preinit /etc/rc5.d/S${PSLEVEL}${HACKNAME}-preinit >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# And make it start at boot, after userstore, after network, after volumd, and before framework (rc5)
logmsg "I" "install" "" "creating boot symlink"
if [ -L /etc/rc5.d/S${SLEVEL}${HACKNAME} ] ; then
    echo "symbolic link /etc/rc5.d/S${SLEVEL}${HACKNAME} already exists, skipping..." >> ${USBNET_LOG}
else
    ln -fs /etc/init.d/${HACKNAME} /etc/rc5.d/S${SLEVEL}${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# Make it stop at reboot (rc6), after the framework and before userstore
logmsg "I" "install" "" "creating reboot runlevel symlink"
if [ -L /etc/rc6.d/K${KLEVEL}${HACKNAME} ] ; then
    echo "symbolic link /etc/rc6.d/K${KLEVEL}${HACKNAME} already exists, skipping..." >> ${USBNET_LOG}
else
    ln -fs /etc/init.d/${HACKNAME} /etc/rc6.d/K${KLEVEL}${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# Make it stop at shutdown (rc0), after the framework and before userstore
logmsg "I" "install" "" "creating shutdown runlevel symlink"
if [ -L /etc/rc0.d/K${KLEVEL}${HACKNAME} ] ; then
    echo "symbolic link /etc/rc0.d/K${KLEVEL}${HACKNAME} already exists, skipping..." >> ${USBNET_LOG}
else
    ln -fs /etc/init.d/${HACKNAME} /etc/rc0.d/K${KLEVEL}${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# Make it stop when updating (rc3), after the framework and before the updater
logmsg "I" "install" "" "creating update runlevel symlink"
if [ -L /etc/rc3.d/K${KLEVEL}${HACKNAME} ] ; then
    echo "symbolic link /etc/rc3.d/K${KLEVEL}${HACKNAME} already exists, skipping..." >> ${USBNET_LOG}
else
    ln -fs /etc/init.d/${HACKNAME} /etc/rc3.d/K${KLEVEL}${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# Cleanup
logmsg "I" "install" "" "cleaning up"
rm -f ${HACKNAME}-preinit ${HACKNAME}-init ${HACKNAME}.tar.xz xzdec

otautils_update_progressbar

echo "Done!" >> ${USBNET_LOG}
logmsg "I" "install" "" "done"

otautils_update_progressbar

return ${OK}
