//
//  kindle_usbnet_addr.c
//  Get the IP address of the usb0 network interface (used by the USBNetwork hack)
//  This is nearly identical to the example from the getifaddrs manpage.
//
//  Copyright (C) 2012 NiLuJe
//
//  $Id: kindle_usbnet_addr.c 8597 2012-09-20 02:25:05Z NiLuJe $
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <arpa/inet.h>
#include <sys/socket.h>
#include <netdb.h>
#include <ifaddrs.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

int main(int argc, char *argv[])
{
	struct ifaddrs *ifaddr, *ifa;
	int family, s;
	char host[NI_MAXHOST];
	unsigned int usb0_up = 0;

	if(getifaddrs(&ifaddr) == -1)
	{
		perror("getifaddrs");
		exit(EXIT_FAILURE);
	}

	/* Walk through linked list, maintaining head pointer so we
	can free list later */
	for(ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next)
	{
		if(ifa->ifa_addr == NULL)
			continue;

		// We only care about usb0
		if(strncmp(ifa->ifa_name, "usb0", 4) != 0)
			continue;

		// We found usb0, w00t
		usb0_up = 1;

		family = ifa->ifa_addr->sa_family;

		// Try to get the IPv4 address of our interface
		if(family == AF_INET)
		{
			s = getnameinfo(ifa->ifa_addr, sizeof(struct sockaddr_in), host, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
			if(s != 0)
			{
				printf("getnameinfo() failed: %s\n", gai_strerror(s));
				exit(EXIT_FAILURE);
			}
			printf("%s address: <%s>\n", ifa->ifa_name, host);
		}
	}

	freeifaddrs(ifaddr);

	// If usb0 wasn't up, go away
	if(! usb0_up)
	{
		printf("couldn't find an active usb0 network interface\n");
		exit(EXIT_FAILURE);
	}
	else
	{
		exit(EXIT_SUCCESS);
	}
}
