#!/bin/sh
#
# $Id: uninstall.sh 18462 2021-04-22 17:44:08Z NiLuJe $
#

# Pull libOTAUtils for logging & progress handling
[ -f ./libotautils ] && source ./libotautils


# Hack specific config (name and when to start/stop)
HACKNAME="usbnet"
PSLEVEL="62"
SLEVEL="82"
KLEVEL="09"

# Based on version 0.11 (20100124), ebs, jya
HACKVER="0.57.N"

# Directories
USBNET_BASEDIR="/mnt/us/usbnet"
USBNET_BINDIR="${USBNET_BASEDIR}/bin"
USBNET_SBINDIR="${USBNET_BASEDIR}/sbin"
USBNET_LIBEDIR="${USBNET_BASEDIR}/libexec"

USBNET_LOG="${USBNET_BASEDIR}/usbnetwork_uninstall.log"

if grep -q '^Kindle 4' /etc/prettyversion.txt ; then
    IS_K4="true"
else
    IS_K4="false"
fi

# The K4 looks for usbnetwork in /usr/local/bin ...
if [ "${IS_K4}" == "true" ] ; then
    KINDLE_TESTDIR="/usr/local/bin"
else
    KINDLE_TESTDIR="/test/bin"
fi
KINDLE_USBNETBIN="${KINDLE_TESTDIR}/usbnetwork"

USBNET_USBNETBIN="${USBNET_BINDIR}/usbnetwork"

# Result codes
OK=0
ERR=${OK}

# Here we go
echo >> ${USBNET_LOG}
echo "usbnetwork v${HACKVER}, $( date )" >> ${USBNET_LOG}

otautils_update_progressbar

# Remove our deprecated content
# From v0.21.N
logmsg "I" "uninstall" "" "removing deprecated init scripts & symlinks (v0.21.N)"
if [ -f /etc/init.d/${HACKNAME}-wifi ] ; then
    echo "/etc/init.d/${HACKNAME}-wifi exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/init.d/${HACKNAME}-wifi >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

if [ -L /etc/rc5.d/S72${HACKNAME}-wifi ] ; then
    echo "symbolic link /etc/rc5.d/S72${HACKNAME}-wifi exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/rc5.d/S72${HACKNAME}-wifi >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# From v0.22.N
logmsg "I" "uninstall" "" "removing deprecated symlinks (v0.22.N)"
if [ -L /etc/rcS.d/S72${HACKNAME} ] ; then
    echo "symbolic link /etc/rcS.d/S72${HACKNAME} exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/rcS.d/S72${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# From v0.23.N
logmsg "I" "uninstall" "" "removing deprecated symlinks (v0.23.N)"
if [ -L /etc/rc5.d/S99${HACKNAME} ] ; then
    echo "symbolic link /etc/rc5.d/S99${HACKNAME} exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/rc5.d/S99${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# Remove `usbNetwork command symlink
logmsg "I" "uninstall" "" "removing usbNetwork private command symlink"
if [ -L ${KINDLE_USBNETBIN} ] ; then
    echo "symbolic link ${KINDLE_USBNETBIN} -> $( readlink ${KINDLE_USBNETBIN} ) exists, deleting..." >> ${USBNET_LOG}
    rm -f ${KINDLE_USBNETBIN} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# Remove SSH server symlinks
logmsg "I" "uninstall" "" "removing SSH server symlinks"
LIST="/usr/sbin/dropbearmulti /usr/bin/dropbear /usr/bin/dbclient /usr/bin/dropbearkey /usr/bin/dropbearconvert /usr/bin/dbscp /usr/bin/scp"
for var in ${LIST} ; do
    if [ -L ${var} ] ; then
        echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
        DBM=$( readlink ${var} )
        if [ "${DBM}" = "${USBNET_BINDIR}/dropbearmulti" ] ; then
            rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove lsof symlink
logmsg "I" "uninstall" "" "removing lsof symlink"
var="/usr/sbin/lsof"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/lsof" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Remove htop symlink
logmsg "I" "uninstall" "" "removing htop symlink"
var="/usr/bin/htop"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/htop" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Remove rsync symlinks
logmsg "I" "uninstall" "" "removing rsync symlinks"
LIST="rsync rsync-ssl"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove mosh symlink
logmsg "I" "uninstall" "" "removing mosh (server) symlink"
var="/usr/bin/mosh-server"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/mosh-server" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

logmsg "I" "uninstall" "" "removing mosh (client) symlink"
var="/usr/bin/mosh-client"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/mosh-client" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Remove kindletool symlink
logmsg "I" "uninstall" "" "removing kindletool symlink"
var="/usr/bin/kindletool"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/kindletool" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Remove fbgrab symlink
logmsg "I" "uninstall" "" "removing fbgrab symlink"
var="/usr/bin/fbgrab"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/fbgrab" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Remove sshd symlink
logmsg "I" "uninstall" "" "removing sshd symlink (deprecated)"
var="/usr/sbin/sshd"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/sshd" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Remove sshd symlink
logmsg "I" "uninstall" "" "removing sshd symlink"
var="/usr/sbin/sshd"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_SBINDIR}/sshd" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Remove SSH client & utils symlinks
logmsg "I" "uninstall" "" "removing SSH client & utils symlinks"
LIST="scp sftp ssh ssh-add ssh-agent ssh-keygen ssh-keyscan sshfs"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove strace, ltrace & elfutils tools symlinks
logmsg "I" "uninstall" "" "removing strace and friends symlinks"
LIST="strace eu-nm eu-objdump eu-readelf eu-strings ltrace readelf"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove nano symlink
logmsg "I" "uninstall" "" "removing nano symlink"
var="/usr/bin/nano"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/nano" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Remove zsh symlink
logmsg "I" "uninstall" "" "removing zsh symlink"
LIST="/bin/zsh /usr/bin/zsh"
for var in ${LIST} ; do
    if [ -L ${var} ] ; then
        echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink ${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/zsh" ] ; then
            rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove ag symlink
logmsg "I" "uninstall" "" "removing ag symlink"
var="/usr/bin/ag"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/ag" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Remove tmux symlink
logmsg "I" "uninstall" "" "removing tmux symlink"
var="/usr/bin/tmux"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/tmux" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Remove objdump symlink
logmsg "I" "uninstall" "" "removing objdump symlink"
var="/usr/bin/objdump"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/objdump" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Remove gprof symlink
logmsg "I" "uninstall" "" "removing gprof symlink"
var="/usr/bin/gprof"
if [ -L ${var} ] ; then
    echo "symbolic link ${var} -> $( readlink ${var} ) exists, deleting..." >> ${USBNET_LOG}
    SYMBIN=$( readlink ${var} )
    if [ "${SYMBIN}" = "${USBNET_BINDIR}/gprof" ] ; then
        rm -f ${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    else
        echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
    fi
fi

otautils_update_progressbar

# Remove fbink symlink
logmsg "I" "uninstall" "" "removing fbink symlink"
LIST="fbink fbdepth"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove evemu symlinks
logmsg "I" "uninstall" "" "removing evemu symlinks"
LIST="evemu-describe evemu-device evemu-event evemu-play evemu-record"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove pax-utils symlinks
logmsg "I" "uninstall" "" "removing pax-utils symlinks"
LIST="scanelf lddtree symtree"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove util-linux symlinks
logmsg "I" "uninstall" "" "removing util-linux symlinks"
LIST="lsblk choom lsipc lslocks"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

LIST="blkid"
for var in ${LIST} ; do
    if [ -L /sbin/${var} ] ; then
        echo "symbolic link /sbin/${var} -> $( readlink /sbin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /sbin/${var} )
        if [ "${SYMBIN}" = "${USBNET_SBINDIR}/${var}" ] ; then
            rm -f /sbin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove jq symlinks
logmsg "I" "uninstall" "" "removing jq symlinks"
LIST="jq"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove bsdtar symlinks
logmsg "I" "uninstall" "" "removing bsdtar symlinks"
LIST="bsdtar"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove lftp symlinks
logmsg "I" "uninstall" "" "removing lftp symlinks"
LIST="lftp lftpget"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove tree symlinks
logmsg "I" "uninstall" "" "removing tree symlinks"
LIST="tree"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove b2sum symlinks
logmsg "I" "uninstall" "" "removing b2sum symlinks"
LIST="b2sum"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Remove file symlinks
logmsg "I" "uninstall" "" "removing file symlinks"
LIST="file"
for var in ${LIST} ; do
    if [ -L /usr/bin/${var} ] ; then
        echo "symbolic link /usr/bin/${var} -> $( readlink /usr/bin/${var} ) exists, deleting..." >> ${USBNET_LOG}
        SYMBIN=$( readlink /usr/bin/${var} )
        if [ "${SYMBIN}" = "${USBNET_BINDIR}/${var}" ] ; then
            rm -f /usr/bin/${var} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
        else
            echo "symbolic link is not ours, skipping..." >> ${USBNET_LOG}
        fi
    fi
done

otautils_update_progressbar

# Delete init script
logmsg "I" "uninstall" "" "removing preinit scripts"
if [ -f /etc/init.d/${HACKNAME}-preinit ] ; then
    echo "/etc/init.d/${HACKNAME}-preinit exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/init.d/${HACKNAME}-preinit >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

logmsg "I" "uninstall" "" "removing init scripts"
if [ -f /etc/init.d/${HACKNAME} ] ; then
    echo "/etc/init.d/${HACKNAME} exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/init.d/${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

logmsg "I" "uninstall" "" "removing preinit symlink"
if [ -L /etc/rc5.d/S${PSLEVEL}${HACKNAME}-preinit ] ; then
    echo "symbolic link /etc/rc5.d/S${PSLEVEL}${HACKNAME}-preinit exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/rc5.d/S${PSLEVEL}${HACKNAME}-preinit >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

logmsg "I" "uninstall" "" "removing boot runlevel symlink"
if [ -L /etc/rc5.d/S${SLEVEL}${HACKNAME} ] ; then
    echo "symbolic link /etc/rc5.d/S${SLEVEL}${HACKNAME} exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/rc5.d/S${SLEVEL}${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

logmsg "I" "uninstall" "" "removing reboot runlevel symlink"
if [ -L /etc/rc6.d/K${KLEVEL}${HACKNAME} ] ; then
    echo "symbolic link /etc/rc6.d/K${KLEVEL}${HACKNAME} exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/rc6.d/K${KLEVEL}${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

logmsg "I" "uninstall" "" "removing shutdown runlevel symlink"
if [ -L /etc/rc0.d/K${KLEVEL}${HACKNAME} ] ; then
    echo "symbolic link /etc/rc0.d/K${KLEVEL}${HACKNAME} exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/rc0.d/K${KLEVEL}${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

logmsg "I" "uninstall" "" "removing update runlevel symlink"
if [ -L /etc/rc3.d/K${KLEVEL}${HACKNAME} ] ; then
    echo "symbolic link /etc/rc3.d/K${KLEVEL}${HACKNAME} exists, deleting..." >> ${USBNET_LOG}
    rm -f /etc/rc3.d/K${KLEVEL}${HACKNAME} >> ${USBNET_LOG} 2>&1 || exit ${ERR}
fi

otautils_update_progressbar

# Disable USB network tethering
logmsg "I" "uninstall" "" "removing traces of USB network tethering"
echo "Part I done! Now to remove traces of USB network tethering..." >> ${USBNET_LOG}

# Handle K3?
for platform in mario luigi ; do
    fwk_cfg="/opt/amazon/ebook/config/framework.${platform}.conf"
    if [ -f "${fwk_cfg}" ] ; then
        logmsg I "update" "restoring ${platform} config"
        sed -e "s/USE_WAN : false/USE_WAN : true/g" -i "${fwk_cfg}" >> ${USBNET_LOG} 2>&1 || exit ${ERR}
    fi
done

rm -f /etc/resolv.d/resolv.conf.default >> ${USBNET_LOG} 2>&1 || exit ${ERR}
touch /etc/resolv.d/resolv.conf.default >> ${USBNET_LOG} 2>&1 || exit ${ERR}
cp -f /etc/resolv.d/resolv.conf.default /var/run/resolv.conf >> ${USBNET_LOG} 2>&1 || exit ${ERR}

echo "All done!" >> ${USBNET_LOG}

otautils_update_progressbar

# From v0.11.N
# Remove custom directory in userstore?
logmsg "I" "uninstall" "" "removing kual extension"
if [ -d /mnt/us/extensions/${HACKNAME} ] ; then
    rm -rf /mnt/us/extensions/${HACKNAME}
    logmsg "I" "uninstall" "" "kual extension has been removed"
fi
logmsg "I" "uninstall" "" "removing custom directory (only if /mnt/us/${HACKNAME}/uninstall exists)"
if [ -d /mnt/us/${HACKNAME} -a -f /mnt/us/${HACKNAME}/uninstall ] ; then
    rm -rf /mnt/us/${HACKNAME}
    logmsg "I" "uninstall" "" "custom directory has been removed"
fi

otautils_update_progressbar

logmsg "I" "uninstall" "" "done"

otautils_update_progressbar

return ${OK}
